import os
from os import mkdir
from os.path import isdir, join, dirname, abspath

# Test if the application installed on the system
_installed = "site-packages" in __file__ or "python-support" in __file__

def isInstalled():
    return _installed

# Helper function
def _joinDir(dir, *subpaths):
    if not subpaths:
        return dir + "/"
    else:
        return join(dir, *subpaths)

_prefixes = ("/usr/share", "/usr/local/share")

# Test if we are installed on the system, or are being run from tar/svn
if isInstalled():
    for prefix in _prefixes:
        if isdir(_joinDir(prefix, "lottanzb")):
            _prefix = _joinDir(prefix, "lottanzb")
            break
else:
    _prefix = abspath(_joinDir(dirname(__file__), "../"))

# The the root directory
def dataDir(*subpaths):
    return _joinDir(_prefix, *subpaths)

_gladeDir = _joinDir(_prefix, "glade")

def gladeDir(*subpaths):
    return _joinDir(_gladeDir, *subpaths)

# Determine the user's home directory path
if os.name == "posix":
    _home = os.environ.get("HOME")
elif os.name == "nt":
    _home = os.environ.get("APPDATA")

def homeDir(*subpaths):
    return _joinDir(_home, *subpaths)

# Where to store the HellaNZB and LottaNZB configuration files
_clientDir = homeDir(".lottanzb")

def clientDir(*subpaths):
    return _joinDir(_clientDir, *subpaths)

def hasClientDir():
    return isdir(_clientDir)

def makeClientDir():
    if not hasClientDir():
        os.mkdir(_clientDir)

# Where place to store temporary files
if os.name == "posix":
    _temp = clientDir("tmp")
elif os.name == "nt":
    _temp = os.environ.get("TMP")

def tempDir(*subpaths):
    return _joinDir(_temp, *subpaths)

def makeTempDir():
    if not isdir(tempDir()):
        mkdir(tempDir())

makeClientDir()
makeTempDir()

if isInstalled():
    _docsDir = "/usr/share/gnome/help/lottanzb"
else:
    _docsDir = _joinDir(_prefix, "help")

def docsDir(*subpaths):
    return _joinDir(_docsDir, *subpaths)
