import logging, os.path

from gobject import GObject, property, list_properties
from os import getcwd
from os.path import isfile, join
from paths import clientDir, homeDir

class HellaPrefs(GObject):
    PREFIX_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb"), \
        nick    = "Directory prefix", \
        blurb   = "This property doesn't have any effect in stand-alone mode.")
    
    QUEUE_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.queue"), \
        nick    = "Queue directory", \
        blurb   = "Queued .nzb files are stored here.")
    
    DEST_DIR = property( \
        type    = str, \
        default = homeDir("Downloads"), \
        nick    = "Download directory", \
        blurb   = "Completed downloads go here. Depending on your configuration, the downloaded archives will already be validated and extracted.")
    
    CURRENT_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.current"), \
        nick    = "Current download directory", \
        blurb   = "The NZB file currently being downloaded is stored here.")
    
    WORKING_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.working"), \
        nick    = "Working directory", \
        blurb   = "The archive currently being downloaded is stored here.")
    
    POSTPONED_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.postponed"), \
        nick    = "Directory containing postponed downloads", \
        blurb   = "Archives interrupted in the middle of downloading are stored here temporarily.")
    
    PROCESSING_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.processing"), \
        nick    = "Processing directory", \
        blurb   = "Archives currently being processed are stored here. It may contains archive directories or symbolic links to archive directories.")
    
    PROCESSED_SUBDIR = property( \
        type    = str, \
        default = "processed", \
        nick    = "Directory containing processed files", \
        blurb   = "Sub directory within the NZB archive directory to move processed files to.")
    
    TEMP_DIR = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "daemon.temp"), \
        nick    = "Temporary storage directory")
    
    MAX_RATE = property( \
        type    = int, \
        nick    = "Maximum download speed", \
        blurb   = "Limit all server connections to the specified KB/s.")
    
    STATE_XML_FILE = property( \
        type    = str, \
        default = homeDir(".hellanzb", "nzb", "hellanzbState.xml"), \
        nick    = "Location of HellaNZB's state file", \
        blurb   = "This file is used to store HellaNZB's state when HellaNZB isn't running. The state is intermittently written out as XML to this file. It includes the order of the queue, and SmartPAR recovery information.")
    
    DEBUG_MODE = property( \
        type    = str, \
        nick    = "Debug log file", \
        blurb   = "Log debug messages to the specified file.")
    
    LOG_FILE = property( \
        type    = str, \
        default = homeDir(".hellanzb", "log"), \
        nick    = "Log file", \
        blurb   = "Log output to the specified file. Set to None for no logging.")
    
    LOG_FILE_MAX_BYTES = property( \
        type    = long, \
        nick    = "Maximum log file size", \
        blurb   = "Automatically roll over both log files when they reach LOG_FILE_MAX_BYTES size.")
    
    LOG_FILE_BACKUP_COUNT = property( \
        type    = int, \
        nick    = "Number of log files to be backed up")
    
    DELETE_PROCESSED = property( \
        type    = bool, \
        default = True, \
        nick    = "Remove the PROCESSED_SUBDIR if the archive was successfully post-processed", \
        blurb   = "Warning: The normal LOG_FILE should be enabled with this option - for a record of what hellanzb deletes.")
    
    CACHE_LIMIT = property( \
        type    = str, \
        default = "0", \
        nick    = "Maximum amount of memory used to cache encoded article data segments", \
        blurb   = "HellaNZB will write article data to disk when this cache is exceeded. Available settings: -1 for unlimited size, 0 to disable cache (only cache to disk), > 0 to limit cache to this size, in bytes, KB, MB, e.g. 1024 '1024KB' '100MB' '1GB'.")
    
    SMART_PAR = property( \
        type    = bool, \
        default = True, \
        nick    = "Download PARs only if necessary")
    
    SKIP_UNRAR = property( \
        type    = bool, \
        default = False, \
        nick    = "Skip unraring during post-processing")
    
    UNRAR_CMD = property( \
        type    = str, \
        default = "/usr/bin/unrar", \
        nick    = "Path to the (un)rar command")
    
    PAR2_CMD = property( \
        type    = str, \
        default = "/usr/bin/par2", \
        nick    = "Path to the par2 command")
    
    UMASK = property( \
        type    = int, \
        nick    = "Force umask", \
        blurb   = "HellaNZB inherits the umask from the current user's environment (unless it's running in daemon mode).")
    
    MAX_DECOMPRESSION_THREADS = property( \
        type    = int, \
        default = 2, \
        nick    = "Max files to decompress at the same time")
    
    XMLRPC_SERVER = property( \
        type    = str, \
        default = "localhost", \
        nick    = "XML RPC hostname", \
        blurb   = "Hostname for the XML RPC client to connect to. Defaults to 'localhost'.")
    
    XMLRPC_PASSWORD = property( \
        type    = str, \
        default = "changeme", \
        nick    = "XML RPC password", \
        blurb   = "You might probably never use this, but the command line XML RPC calls do - it should definitely be changed from its default value. The XML RPC username is hardcoded as 'hellanzb'.")
    
    XMLRPC_PORT = property( \
        type    = int, \
        default = 8760, \
        nick    = "XML RPC port number", \
        blurb   = "Port number the XML RPC server will listen on, and the client will connect to. None for no XML RPC server.")
    
    XMLRPC_SERVER_BIND = property( \
        type    = str, \
        default = "127.0.0.1", \
        nick    = "Bind XML RPC server to IP address", \
        blurb   = "IP address on which the XML RPC server will be bound to. '0.0.0.0' for any interfaces, '127.0.0.1' will disable remote access.")
    
    NEWZBIN_USERNAME = property( \
        type    = str, \
        nick    = "newzbin.com username", \
        blurb   = "Username to www.newzbin.com for automatic NZB downloading")
    
    NEWZBIN_PASSWORD = property( \
        type    = str, \
        nick    = "newzbin.com password", \
        blurb   = "Password to www.newzbin.com for automatic NZB downloading")
    
    CATEGORIZE_DEST = property( \
        type    = bool, \
        default = True, \
        nick    = "Categorize newzbin.com downloads", \
        blurb   = "Save archives into a sub directory of DEST_DIR named after their newzbin.com category (when queued using the enqueuenewzbin XML RPC call); e.g. Apps, Movies, Music")
    
    MACBINCONV_CMD = property( \
        type    = str, \
        nick    = "Path to the optional macbinconv command", \
        blurb   = "This command is used to convert MacBinary files")
    
    GROWL_NOTIFY = property( \
        type    = bool, \
        default = False, \
        nick    = "Enable Mac OS X Growl notifications")
    
    GROWL_SERVER = property( \
        type    = str, \
        default = "IP", \
        nick    = "growl notification server")
    
    GROWL_PASSWORD = property( \
        type    = str, \
        default = "password", \
        nick    = "growl password")
    
    LIBNOTIFY_NOTIFY = property( \
        type    = bool, \
        default = False, \
        nick    = "Enable libnotify daemon notifications")
    
    DISABLE_COLORS = property( \
        type    = bool, \
        default = False, \
        nick    = "Disable ANSI color codes in the main screen", \
        blurb   = "Preserves the in place scroller.")
    
    DISABLE_ANSI = property( \
        type    = bool, \
        default = False, \
        nick    = "Disable ALL ANSI color codes in the main screen", \
        blurb   = "For terminals that don't support ANY ANSI codes.")
    
    NZB_ZIPS = property( \
        type    = str, \
        default = ".nzb.zip", \
        nick    = "Support extracting NZBs from ZIP files with this suffix in QUEUE_DIR", \
        blurb   = "Defaults to '.nzb.zip'. Set to False to disable. Case insensitive.")
    
    NZB_GZIPS = property( \
        type    = str, \
        default = ".nzb.gz", \
        nick    = "Support extracting NZBs from GZIP files with this suffix in QUEUE_DIR", \
        blurb   = "Defaults to '.nzb.gz'. Set to False to disable. Case insensitive.")
    
    EXTERNAL_HANDLER_SCRIPT = property( \
        type    = str, \
        nick    = "Optional external handler script", \
        blurb   = "HellaNZB will run this script after post processing an archive.")
    
    NZBQUEUE_MDELAY = property( \
        type    = float, \
        default = 10.0, \
        nick    = "NZB queue delay", \
        blurb   = "Delay enqueueing new, recently modified NZB files added to the QUEUE_DIR until this many seconds have passed since the NZB's last modification time. Defaults to 10 seconds.")
    
    KEEP_FILE_TYPES = property( \
        type    = object, \
        nick    = "File types to keep", \
        blurb   = "Don't get rid of (move into the PROCESSED dir) these file types when finished post processing. Case insensitive.")
    
    OTHER_NZB_FILE_TYPES = property( \
        type    = object, \
        nick    = "Alternative NZB file extensions", \
        blurb   = "List of alternative file extensions matched as NZB files in the QUEUE_DIR. The 'nzb' file extension is always matched.")
    
    NOT_REQUIRED_FILE_TYPES = property( \
        type    = object, \
        nick    = "Not required file types", \
        blurb   = "If any of the following file types are missing from the archive and cannot be repaired, continue processing because they're unimportant. Case insensitive.")
    
    def __init__(self, confFile = None):
        self.servers = []
        self.music_types = []
        
        self.loaded = False
        self.read_only = False
        self.confFile = confFile
        
        GObject.__init__(self)
        
        # GObject properties of type 'object' can't have default values for some reason
        self.KEEP_FILE_TYPES = ["nfo", "txt"]
        self.OTHER_NZB_FILE_TYPES = []
        self.NOT_REQUIRED_FILE_TYPES = ["log", "m3u", "nfo", "nzb", "sfv", "txt"]
        
        self.load()
    
    def __eq__(self, other):
        return not self.__ne__(other)
    
    def __ne__(self, other):
        changed_values = [key for key in self.list_keys() if self.get_property(key) != other.get_property(key)]
        
        return changed_values or \
            self.servers != other.servers or \
            self.music_types != other.music_types
    
    def list_keys(self):
        return [prop.name.replace("-", "_") for prop in list_properties(self)]
    
    def get_property(self, key):
        '''There are some HellaNZB configuration values which can be either
           integer or string for example. This method ensures that those
           properties are correctly saved.'''
        
        value = GObject.get_property(self, key)
        
        if key in ["NEWZBIN_USERNAME", "NEWZBIN_PASSWORD", "MACBINCONV_CMD"] and not value:
            return None
        elif key in ["NZB_ZIPS", "NZB_GZIPS"] and value == "False":
            return False
        elif key in ["CACHE_LIMIT"]:
            try:
                return int(value)
            except:
                pass
        
        return value
    
    def load(self):
        if self.confFile:
            def defineServer(**kwargs):
                self.servers.append(Server(**kwargs))
            
            def defineMusicType(*args):
                self.music_types.append(MusicType(*args))
            
            # Syntax sugar. *g*
            Hellanzb = self
            
            # Todo: More error handlers?
            try:
                execfile(self.confFile)
            except IOError:
                logging.error(_("The specified HellaNZB configuration file doesn't exist."))
            except SyntaxError:
                logging.error(_("The specified HellaNZB configuration file is invalid."))
            else:
                self.loaded = True
    
    def addServer(self, server):
        if not isinstance(server, Server):
            server = Server(**server)
        
        self.servers.append(server)
    
    def hasServers(self):
        return len(self.servers) > 0
    
    def save(self):
        if not self.loaded or self.read_only:
            return
        
        confFile = open(self.confFile, "w")
        confFile.write(self.getConfigStr())
        confFile.close()
        
        logging.debug(_("HellaNZB preferences saved."))
    
    # Todo: Better method name
    def getConfigStr(self):
        output = "# HellaNZB configuration file - Managed by LottaNZB\n\n"
        
        for server in self.servers:
            output += server.getConfigStr()
        
        for music_type in self.music_types:
            output += music_type.getConfigStr()
        
        # These preferences should be commented out 'if not value:'
        deactivable = ["EXTERNAL_HANDLER_SCRIPT", "MACBINCONV_CMD", "UMASK", "UNRAR_CMD", "DEBUG_MODE"]
        
        for property in list_properties(self):
            key = property.name.replace("-", "_")
            value = self.get_property(key)
            prefix = ""
            
            if key in deactivable and not value:
                prefix = "# "
            
            if type(value) == str:
                value = "\"" + value + "\""
            
            output += "\n# " + property.nick + "\n"
            
            if property.blurb:
                output += "# " + property.blurb + "\n"
            
            output += prefix + "Hellanzb." + key + " = " + str(value) + "\n"
        
        return output
    
    def update(self, preferences):
        self.set_properties(**preferences)
    
    def hasNewzBinSupport(self):
        return bool(self.NEWZBIN_USERNAME and self.NEWZBIN_PASSWORD)
    
    @staticmethod
    def locate():
        places = [
            clientDir(),
            getcwd(),
            join(getcwd(), "etc"),
            homeDir(".hellanzb"),
            "/etc",
            join("/etc", "hellanzb")
        ]
        
        for place in places:
            file = join(place, "hellanzb.conf")
            
            if isfile(file):
                return file

class Server(GObject):
    id          = property(type = str)
    username    = property(type = str)
    password    = property(type = str)
    antiIdle    = property(type = int,  default = 270,  minimum = 0)
    connections = property(type = int,  default = 8,    minimum = 1)
    ssl         = property(type = bool, default = False)
    
    _hosts = []
    
    def _get_hosts(self):
        return self._hosts
    
    def _set_hosts(self, hosts):
        def toHost(host):
            if not isinstance(host, Host):
                return Host(host)
            
            return host
        
        self._hosts = map(toHost, hosts)
    
    hosts = property(type = object, getter = _get_hosts, setter = _set_hosts)
    
    def __init__(self, **info):
        GObject.__init__(self)
        
        self.hosts = []
        self.update(info)
    
    def __cmp__(self, other):
        return cmp(self.getConfigStr(), other.getConfigStr())
    
    def getConfigStr(self):
        template = "defineServer(id = %s, username = %s, password = %s, " + \
            "antiIdle = %s, connections = %s, ssl = %s, hosts = %s)\n"
        
        hosts = [str(host) for host in self.hosts]
        
        def noneify(value):
            if not value:
                return None
            else:
                return value
        
        def escape(value):
            if type(value) == str:
                return "\"" + value + "\""
            
            return value
        
        return template % (escape(self.id), escape(noneify(self.username)), escape(noneify(self.password)), self.antiIdle, self.connections, self.ssl, hosts)
    
    def update(self, info):
        self.set_properties(**info)
    
    def getDefaultHost(self):
        return self.hosts[0]
    
    def needsAuthentication(self):
        return bool(self.username and self.password)

class Host(GObject):
    address = property(type = str)
    port    = property(type = int, default = 119, minimum = 1)
    
    def __init__(self, address, port = 119):
        GObject.__init__(self)
        
        if ":" in address:
            self.set_uri(address)
        else:
            self.address = address
            self.port = port
    
    def __str__(self):
        return self.get_uri()
    
    def get_uri(self):
        return "%s:%s" % (self.address, self.port)
    
    def set_uri(self, uri):
        try:
            parts = uri.split(":")
            
            self.address = parts[0]
            self.port = int(parts[1])
        except:
            pass

class MusicType:
    '''Defines a music file type, and whether or not HellaNZB should attempt to
       decompress the music if it comes across this type of file.'''
    
    def __init__(self, extension, decompressor, decompressToType):
        self.extension = extension
        self.decompressor = decompressor
        self.decompressToType = decompressToType
    
    def __cmp__(self, other):
        return cmp(self.extension, other.extension)
    
    def __str__(self):
        return "<MusicType: %s>" % self.extension
    
    def shouldDecompress(self):
        return self.decompressor is not None
    
    def getConfigStr(self):
        def escape(value):
            if type(value) == str:
                return "\"" + value + "\""
            
            return value
        
        return "defineMusicType(%s, %s, %s)\n" % ( \
            escape(self.extension), \
            escape(self.decompressor), \
            escape(self.decompressToType))

try:
    # For Python 2.5
    from xml.etree.ElementTree import ElementTree, Element, SubElement
except:
    # For Python 2.4
    from elementtree.ElementTree import ElementTree, Element, SubElement

class LottaPrefs(dict):
    def __init__(self):
        dict.__init__(self)
        
        self.loaded = False
        self.changeConfFile(clientDir("lottanzb.xml"))
    
    def changeConfFile(self, confFile = None):
        self.confFile = confFile
        
        if self.confFile:
            self.load()
    
    def __getitem__(self, key):
        if key in self:
            return dict.__getitem__(self, key)
    
    def load(self):
        if isfile(self.confFile):
            self.loaded = False
            self.clear()
            
            try:
                tree = ElementTree(file=self.confFile)
                root = tree.getroot()
            except:
                logging.error(_("Could not load the LottaNZB configuration file as it seems to be invalid."))
                return False
            
            for key in list(root):
                value = key.text
                
                if key.tag in LottaPrefs.BOOL_PREFS:
                    value = value == "True"
                elif key.tag in LottaPrefs.INT_PREFS:
                    value = int(value)
                elif key.tag in LottaPrefs.FLOAT_PREFS:
                    value = float(value)
                
                self[key.tag] = value
            
            self.validate()
            
            self.loaded = True
        else:
            logging.info(_("The LottaNZB configuration file doesn't exist."))
    
    def save(self):
        root = Element("lottaprefs")
        
        for key, value in self.iteritems():
            subElement = SubElement(root, key)
            subElement.text = str(self[key])
        
        tree = ElementTree(root)
        tree.write(self.confFile, "utf-8")
        
        self.loaded = True
        
        logging.debug(_("LottaNZB preferences saved."))
    
    def validate(self):
        """Validates preferences and cleanup any common invalid entries"""

        if self["sleep_time"] < 1 or None:
            logging.info("Sleep time setting doesn't exist or is lower than 1 second, settings to 1 second.")
            self["sleep_time"] = 1
    
    STRING_PREFS = ["resume_command", "start_command", "stop_command", "xmlrpc_address", "xmlrpc_password", "frontend_config_file"]
    BOOL_PREFS   = ["remote", "start_minimized", "frontend_mode", "newzbin"]
    INT_PREFS    = ["prefs_revision", "window_height", "window_width", "xmlrpc_port"]
    FLOAT_PREFS  = ["sleep_time"]
    
    # Will be used in future release
    REVISION = 1

lotta = LottaPrefs()
hella = HellaPrefs()
